@@ -18,7 +18,7 @@
 -- this is needed because there is no ALTER EXTENSION DROP FUNCTION/AGGREGATE command
 -- and we can't CREATE OR REPALCe functions whose signatures have changed and we can drop them if they are part of an extention
 -- So we use this to remove it from extension first before we drop
-CREATE FUNCTION postgis_extension_remove_objects(param_extension text, param_type text)
+CREATE OR REPLACE FUNCTION postgis_extension_remove_objects(param_extension text, param_type text)
   RETURNS boolean AS
 $$
 DECLARE
