@@ -76,8 +76,8 @@ static void lwprint_normalize_latlon(double *lat, double *lon)
 static char * lwdouble_to_dms(double val, const char *pos_dir_symbol, const char *neg_dir_symbol, const char * format)
 {
 	/* 3 numbers, 1 sign or compass dir, and 5 possible strings (degree signs, spaces, misc text, etc) between or around them.*/
-	static int NUM_PIECES = 9;
-	static int WORK_SIZE = 1024;
+#	define NUM_PIECES 9
+#	define WORK_SIZE 1024
 	char pieces[NUM_PIECES][WORK_SIZE];
 	int current_piece = 0;
 	int is_negative = 0;
@@ -354,7 +354,7 @@ static char * lwdouble_to_dms(double val, const char *pos_dir_symbol, const char
 	}
 	if(deg_piece >= 0)
 	{
-		sprintf(pieces[deg_piece], "%*.*f", deg_digits, deg_dec_digits, degrees);
+		snprintf(pieces[deg_piece], WORK_SIZE, "%*.*f", deg_digits, deg_dec_digits, degrees);
 	}
 
 	if (min_piece >= 0)
@@ -364,7 +364,7 @@ static char * lwdouble_to_dms(double val, const char *pos_dir_symbol, const char
 		{
 			lwerror("Bad format, minutes (MM.MMM) number of digits was greater than our working limit.");
 		}
-		sprintf(pieces[min_piece], "%*.*f", min_digits, min_dec_digits, minutes);
+		snprintf(pieces[min_piece], WORK_SIZE, "%*.*f", min_digits, min_dec_digits, minutes);
 	}
 	if (sec_piece >= 0)
 	{
@@ -373,11 +373,13 @@ static char * lwdouble_to_dms(double val, const char *pos_dir_symbol, const char
 		{
 			lwerror("Bad format, seconds (SS.SSS) number of digits was greater than our working limit.");
 		}
-		sprintf(pieces[sec_piece], "%*.*f", sec_digits, sec_dec_digits, seconds);
+		snprintf(pieces[sec_piece], WORK_SIZE, "%*.*f", sec_digits, sec_dec_digits, seconds);
 	}
 
 	/* Allocate space for the result.  Leave plenty of room for excess digits, negative sign, etc.*/
 	result = (char*)lwalloc(format_length + WORK_SIZE);
+	memset(result, 0, format_length + WORK_SIZE);
+
 	/* Append all the pieces together. There may be less than 9, but in that case the rest will be blank. */
 	strcpy(result, pieces[0]);
 	for (index = 1; index < NUM_PIECES; index++)
@@ -399,6 +401,7 @@ static char * lwdoubles_to_latlon(double lat, double lon, const char * format)
 	char * lat_text;
 	char * lon_text;
 	char * result;
+	size_t sz;
 
 	/* Normalize lat/lon to the normal (-90 to 90, -180 to 180) range. */
 	lwprint_normalize_latlon(&lat, &lon);
@@ -407,8 +410,9 @@ static char * lwdoubles_to_latlon(double lat, double lon, const char * format)
 	lon_text = lwdouble_to_dms(lon, "E", "W", format);
 
 	/* lat + lon + a space between + the null terminator. */
-	result = (char*)lwalloc(strlen(lat_text) + strlen(lon_text) + 2);
-	sprintf(result, "%s %s", lat_text, lon_text);
+	sz = strlen(lat_text) + strlen(lon_text) + 2;
+	result = (char*)lwalloc(sz);
+	snprintf(result, sz, "%s %s", lat_text, lon_text);
 	lwfree(lat_text);
 	lwfree(lon_text);
 	return result;
@@ -435,3 +439,76 @@ char* lwpoint_to_latlon(const LWPOINT * pt, const char *format)
 	p = getPoint2d_cp(pt->point, 0);
 	return lwdoubles_to_latlon(p->y, p->x, format);
 }
+
+/*
+ * Removes trailing zeros and dot for a %f formatted number.
+ * Modifies input.
+ */
+static void
+trim_trailing_zeros(char* str)
+{
+	char *ptr, *totrim = NULL;
+	int len;
+	int i;
+
+	LWDEBUGF(3, "input: %s", str);
+
+	ptr = strchr(str, '.');
+	if (!ptr) return; /* no dot, no decimal digits */
+
+	LWDEBUGF(3, "ptr: %s", ptr);
+
+	len = strlen(ptr);
+	for (i = len - 1; i; i--)
+	{
+		if (ptr[i] != '0') break;
+		totrim = &ptr[i];
+	}
+	if (totrim)
+	{
+		if (ptr == totrim - 1)
+			*ptr = '\0';
+		else
+			*totrim = '\0';
+	}
+
+	LWDEBUGF(3, "output: %s", str);
+}
+
+/*
+ * Print an ordinate value using at most the given number of decimal digits
+ *
+ * The actual number of printed decimal digits may be less than the
+ * requested ones if out of significant digits.
+ *
+ * The function will not write more than maxsize bytes, including the
+ * terminating NULL. Returns the number of bytes that would have been
+ * written if there was enough space (excluding terminating NULL).
+ * So a return of ``bufsize'' or more means that the string was
+ * truncated and misses a terminating NULL.
+ *
+ */
+int
+lwprint_double(double d, int maxdd, char* buf, size_t bufsize)
+{
+	double ad = fabs(d);
+	int ndd;
+	int length = 0;
+	if (ad <= FP_TOLERANCE)
+	{
+		d = 0;
+		ad = 0;
+	}
+	if (ad < OUT_MAX_DOUBLE)
+	{
+		ndd = ad < 1 ? 0 : floor(log10(ad)) + 1; /* non-decimal digits */
+		if (maxdd > (OUT_MAX_DOUBLE_PRECISION - ndd)) maxdd -= ndd;
+		length = snprintf(buf, bufsize, "%.*f", maxdd, d);
+	}
+	else
+	{
+		length = snprintf(buf, bufsize, "%g", d);
+	}
+	trim_trailing_zeros(buf);
+	return length;
+}
\ No newline at end of file
