@@ -24,6 +24,7 @@
 
 
 #include "postgres.h"
+#include "catalog/pg_type.h" /* for CSTRINGOID */
 
 #include "../postgis_config.h"
 
@@ -31,16 +32,15 @@
 #include <float.h>
 #include <string.h>
 #include <stdio.h>
-#include <errno.h>
 
-#include "liblwgeom.h"         /* For standard geometry types. */
+#include "liblwgeom.h"                  /* For standard geometry types. */
 #include "liblwgeom_internal.h"         /* For FP comparators. */
-#include "lwgeom_pg.h"       /* For debugging macros. */
-#include "geography.h"	     /* For utility functions. */
+#include "lwgeom_pg.h"                  /* For debugging macros. */
+#include "geography.h"                  /* For utility functions. */
 #include "geography_measurement_trees.h" /* For circ_tree caching */
-#include "lwgeom_transform.h" /* For SRID functions */
+#include "lwgeom_transform.h"            /* For SRID functions */
 
-#if PROJ_GEODESIC
+#ifdef PROJ_GEODESIC
 /* round to 10 nm precision */
 #define INVMINDIST 1.0e8
 #else
@@ -59,6 +59,7 @@ Datum geography_length(PG_FUNCTION_ARGS);
 Datum geography_expand(PG_FUNCTION_ARGS);
 Datum geography_point_outside(PG_FUNCTION_ARGS);
 Datum geography_covers(PG_FUNCTION_ARGS);
+Datum geography_coveredby(PG_FUNCTION_ARGS);
 Datum geography_bestsrid(PG_FUNCTION_ARGS);
 Datum geography_perimeter(PG_FUNCTION_ARGS);
 Datum geography_project(PG_FUNCTION_ARGS);
@@ -82,11 +83,11 @@ Datum geography_distance_knn(PG_FUNCTION_ARGS)
 	g1 = PG_GETARG_GSERIALIZED_P(0);
 	g2 = PG_GETARG_GSERIALIZED_P(1);
 
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
+
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
 
-	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
-
 	/* Set to sphere if requested */
 	if ( ! use_spheroid )
 		s.a = s.b = s.radius;
@@ -153,7 +154,7 @@ Datum geography_distance_uncached(PG_FUNCTION_ARGS)
 	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
 		use_spheroid = PG_GETARG_BOOL(3);
 
-	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
 
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
@@ -214,11 +215,10 @@ Datum geography_distance(PG_FUNCTION_ARGS)
 	g1 = PG_GETARG_GSERIALIZED_P(0);
 	g2 = PG_GETARG_GSERIALIZED_P(1);
 
-	/* Read our calculation type. */
-	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
-		use_spheroid = PG_GETARG_BOOL(3);
+	if (PG_NARGS() > 2)
+		use_spheroid = PG_GETARG_BOOL(2);
 
-	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
 
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
@@ -267,7 +267,6 @@ Datum geography_distance(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT8(distance);
 }
 
-
 /*
 ** geography_dwithin(GSERIALIZED *g1, GSERIALIZED *g2, double tolerance, boolean use_spheroid)
 ** returns double distance in meters
@@ -275,17 +274,15 @@ Datum geography_distance(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(geography_dwithin);
 Datum geography_dwithin(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *g1 = NULL;
-	GSERIALIZED *g2 = NULL;
-	double tolerance = 0.0;
-	double distance;
-	bool use_spheroid = true;
+	GSERIALIZED *g1 = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *g2 = PG_GETARG_GSERIALIZED_P(1);
 	SPHEROID s;
+	double tolerance = 0.0;
+	bool use_spheroid = true;
+	double distance;
 	int dwithin = LW_FALSE;
 
-	/* Get our geometry objects loaded into memory. */
-	g1 = PG_GETARG_GSERIALIZED_P(0);
-	g2 = PG_GETARG_GSERIALIZED_P(1);
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
 
 	/* Read our tolerance value. */
 	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
@@ -295,8 +292,6 @@ Datum geography_dwithin(PG_FUNCTION_ARGS)
 	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
 		use_spheroid = PG_GETARG_BOOL(3);
 
-	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
-
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
 
@@ -306,11 +301,7 @@ Datum geography_dwithin(PG_FUNCTION_ARGS)
 
 	/* Return FALSE on empty arguments. */
 	if ( gserialized_is_empty(g1) || gserialized_is_empty(g2) )
-	{
-		PG_FREE_IF_COPY(g1, 0);
-		PG_FREE_IF_COPY(g2, 1);
 		PG_RETURN_BOOL(false);
-	}
 
 	/* Do the brute force calculation if the cached calculation doesn't tick over */
 	if ( LW_FAILURE == geography_dwithin_cache(fcinfo, g1, g2, &s, tolerance, &dwithin) )
@@ -326,13 +317,18 @@ Datum geography_dwithin(PG_FUNCTION_ARGS)
 		lwgeom_free(lwgeom2);
 	}
 
-	/* Clean up */
 	PG_FREE_IF_COPY(g1, 0);
 	PG_FREE_IF_COPY(g2, 1);
-
 	PG_RETURN_BOOL(dwithin);
 }
 
+PG_FUNCTION_INFO_V1(geography_intersects);
+Datum geography_intersects(PG_FUNCTION_ARGS)
+{
+	PG_RETURN_BOOL(CallerFInfoFunctionCall2(
+		geography_dwithin, fcinfo->flinfo, InvalidOid,
+		PG_GETARG_DATUM(0), PG_GETARG_DATUM(1)));
+}
 
 /*
 ** geography_distance_tree(GSERIALIZED *g1, GSERIALIZED *g2, double tolerance, boolean use_spheroid)
@@ -352,7 +348,9 @@ Datum geography_distance_tree(PG_FUNCTION_ARGS)
 	g1 = PG_GETARG_GSERIALIZED_P(0);
 	g2 = PG_GETARG_GSERIALIZED_P(1);
 
-	/* Return FALSE on empty arguments. */
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
+
+	/* Return zero on empty arguments. */
 	if ( gserialized_is_empty(g1) || gserialized_is_empty(g2) )
 	{
 		PG_FREE_IF_COPY(g1, 0);
@@ -368,8 +366,6 @@ Datum geography_distance_tree(PG_FUNCTION_ARGS)
 	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
 		use_spheroid = PG_GETARG_BOOL(3);
 
-	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
-
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
 
@@ -383,6 +379,9 @@ Datum geography_distance_tree(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
+	/* Knock off any funny business at the nanometer level, ticket #2168 */
+ 	distance = round(distance * INVMINDIST) / INVMINDIST;
+
 	PG_RETURN_FLOAT8(distance);
 }
 
@@ -407,6 +406,7 @@ Datum geography_dwithin_uncached(PG_FUNCTION_ARGS)
 	/* Get our geometry objects loaded into memory. */
 	g1 = PG_GETARG_GSERIALIZED_P(0);
 	g2 = PG_GETARG_GSERIALIZED_P(1);
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
 
 	/* Read our tolerance value. */
 	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
@@ -416,8 +416,6 @@ Datum geography_dwithin_uncached(PG_FUNCTION_ARGS)
 	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
 		use_spheroid = PG_GETARG_BOOL(3);
 
-	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
-
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
 
@@ -532,7 +530,7 @@ Datum geography_area(PG_FUNCTION_ARGS)
 	else
 		lwgeom_calculate_gbox_geodetic(lwgeom, &gbox);
 
-#if ! PROJ_GEODESIC
+#ifndef PROJ_GEODESIC
 	/* Test for cases that are currently not handled by spheroid code */
 	if ( use_spheroid )
 	{
@@ -543,7 +541,7 @@ Datum geography_area(PG_FUNCTION_ARGS)
 		if ( gbox.zmax > 0.0 && gbox.zmin < 0.0 )
 			use_spheroid = LW_FALSE;
 	}
-#endif /* if ! PROJ_GEODESIC */
+#endif /* ifndef PROJ_GEODESIC */
 
 	/* User requests spherical calculation, turn our spheroid into a sphere */
 	if ( ! use_spheroid )
@@ -690,7 +688,6 @@ Datum geography_point_outside(PG_FUNCTION_ARGS)
 	GBOX gbox;
 	GSERIALIZED *g = NULL;
 	GSERIALIZED *g_out = NULL;
-	size_t g_out_size;
 	LWGEOM *lwpoint = NULL;
 	POINT2D pt;
 
@@ -710,12 +707,8 @@ Datum geography_point_outside(PG_FUNCTION_ARGS)
 	gbox_pt_outside(&gbox, &pt);
 
 	lwpoint = (LWGEOM*) lwpoint_make2d(4326, pt.x, pt.y);
-	/* TODO: Investigate where this is used, this was probably not
-	* returning a geography object before. How did this miss checking
-	*/
-	lwgeom_set_geodetic(lwpoint, true);
-	g_out = gserialized_from_lwgeom(lwpoint, &g_out_size);
-	SET_VARSIZE(g_out, g_out_size);
+
+	g_out = geography_serialize(lwpoint);
 
 	PG_FREE_IF_COPY(g, 0);
 	PG_RETURN_POINTER(g_out);
@@ -741,13 +734,12 @@ Datum geography_covers(PG_FUNCTION_ARGS)
 	/* Get our geometry objects loaded into memory. */
 	g1 = PG_GETARG_GSERIALIZED_P(0);
 	g2 = PG_GETARG_GSERIALIZED_P(1);
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
 
 	/* Construct our working geometries */
 	lwgeom1 = lwgeom_from_gserialized(g1);
 	lwgeom2 = lwgeom_from_gserialized(g2);
 
-	error_if_srid_mismatch(lwgeom1->srid, lwgeom2->srid);
-
 	/* EMPTY never intersects with another geometry */
 	if ( lwgeom_is_empty(lwgeom1) || lwgeom_is_empty(lwgeom2) )
 	{
@@ -770,6 +762,47 @@ Datum geography_covers(PG_FUNCTION_ARGS)
 	PG_RETURN_BOOL(result);
 }
 
+PG_FUNCTION_INFO_V1(geography_coveredby);
+Datum geography_coveredby(PG_FUNCTION_ARGS)
+{
+	LWGEOM *lwgeom1 = NULL;
+	LWGEOM *lwgeom2 = NULL;
+	GSERIALIZED *g1 = NULL;
+	GSERIALIZED *g2 = NULL;
+	int result = LW_FALSE;
+
+	/* Get our geometry objects loaded into memory. */
+	/* Pick them up in reverse order to covers */
+	g1 = PG_GETARG_GSERIALIZED_P(1);
+	g2 = PG_GETARG_GSERIALIZED_P(0);
+	gserialized_error_if_srid_mismatch(g1, g2, __func__);
+
+	/* Construct our working geometries */
+	lwgeom1 = lwgeom_from_gserialized(g1);
+	lwgeom2 = lwgeom_from_gserialized(g2);
+
+	/* EMPTY never intersects with another geometry */
+	if ( lwgeom_is_empty(lwgeom1) || lwgeom_is_empty(lwgeom2) )
+	{
+		lwgeom_free(lwgeom1);
+		lwgeom_free(lwgeom2);
+		PG_FREE_IF_COPY(g1, 1);
+		PG_FREE_IF_COPY(g2, 0);
+		PG_RETURN_BOOL(false);
+	}
+
+	/* Calculate answer */
+	result = lwgeom_covers_lwgeom_sphere(lwgeom1, lwgeom2);
+
+	/* Clean up */
+	lwgeom_free(lwgeom1);
+	lwgeom_free(lwgeom2);
+	PG_FREE_IF_COPY(g1, 1);
+	PG_FREE_IF_COPY(g2, 0);
+
+	PG_RETURN_BOOL(result);
+}
+
 /*
 ** geography_bestsrid(GSERIALIZED *g, GSERIALIZED *g) returns int
 ** Utility function. Returns negative SRID numbers that match to the
@@ -788,30 +821,45 @@ Datum geography_bestsrid(PG_FUNCTION_ARGS)
 	int empty2 = LW_FALSE;
 	double xwidth, ywidth;
 	POINT2D center;
-
-	Datum d1 = PG_GETARG_DATUM(0);
-	Datum d2 = PG_GETARG_DATUM(1);
+	LWGEOM *lwgeom;
 
 	/* Get our geometry objects loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(d1);
+	g1 = PG_GETARG_GSERIALIZED_P(0);
 	/* Synchronize our box types */
-	gbox1.flags = g1->flags;
+	gbox1.flags = gserialized_get_lwflags(g1);
 	/* Calculate if the geometry is empty. */
 	empty1 = gserialized_is_empty(g1);
+
+	/* Convert g1 to LWGEOM type */
+	lwgeom = lwgeom_from_gserialized(g1);
+
 	/* Calculate a geocentric bounds for the objects */
 	if ( ! empty1 && gserialized_get_gbox_p(g1, &gbox1) == LW_FAILURE )
 		elog(ERROR, "Error in geography_bestsrid calling gserialized_get_gbox_p(g1, &gbox1)");
 
 	POSTGIS_DEBUGF(4, "calculated gbox = %s", gbox_to_string(&gbox1));
 
+	if ( !lwgeom_isfinite(lwgeom) ) {
+		elog(ERROR, "Error in geography_bestsrid calling with infinite coordinate geographies");
+	}
+	lwgeom_free(lwgeom);
+
 	/* If we have a unique second argument, fill in all the necessary variables. */
-	if ( d1 != d2 )
+	if (PG_NARGS() > 1)
 	{
-		g2 = (GSERIALIZED*)PG_DETOAST_DATUM(d2);
-		gbox2.flags = g2->flags;
+		g2 = PG_GETARG_GSERIALIZED_P(1);
+		gbox2.flags = gserialized_get_lwflags(g2);
 		empty2 = gserialized_is_empty(g2);
 		if ( ! empty2 && gserialized_get_gbox_p(g2, &gbox2) == LW_FAILURE )
 			elog(ERROR, "Error in geography_bestsrid calling gserialized_get_gbox_p(g2, &gbox2)");
+
+		/* Convert g2 to LWGEOM type */
+		lwgeom = lwgeom_from_gserialized(g2);
+
+		if ( !lwgeom_isfinite(lwgeom) ) {
+			elog(ERROR, "Error in geography_bestsrid calling with second arg infinite coordinate geographies");
+		}
+		lwgeom_free(lwgeom);
 	}
 	/*
 	** If no unique second argument, copying the box for the first
