@@ -85,7 +85,7 @@ Datum ST_LocateAlong(PG_FUNCTION_ARGS)
 	GSERIALIZED *gout;
 	LWGEOM *lwin = NULL, *lwout = NULL;
 	double measure = PG_GETARG_FLOAT8(1);
-	double offset = PG_GETARG_FLOAT8(2);;
+	double offset = PG_GETARG_FLOAT8(2);
 
 	lwin = lwgeom_from_gserialized(gin);
 	lwout = lwgeom_locate_along(lwin, measure, offset);
@@ -160,7 +160,7 @@ Datum ST_LocateBetweenElevations(PG_FUNCTION_ARGS)
 
 	if ( ! gserialized_has_z(geom_in) )
 	{
-		elog(ERROR,"This function only accepts LINESTRING or MULTILINESTRING with Z dimensions.");
+		elog(ERROR, "This function only accepts geometries with Z dimensions.");
 		PG_RETURN_NULL();
 	}
 
@@ -195,11 +195,11 @@ Datum ST_InterpolatePoint(PG_FUNCTION_ARGS)
 	}
 	if ( gserialized_get_type(gser_point) != POINTTYPE )
 	{
-		elog(ERROR,"ST_InterpolatePoint: 2st argument isn't a point");
+		elog(ERROR,"ST_InterpolatePoint: 2nd argument isn't a point");
 		PG_RETURN_NULL();
 	}
 
-	error_if_srid_mismatch(gserialized_get_srid(gser_line), gserialized_get_srid(gser_point));
+	gserialized_error_if_srid_mismatch(gser_line, gser_point, __func__);
 
 	if ( ! gserialized_has_m(gser_line) )
 	{
@@ -233,11 +233,11 @@ Datum LWGEOM_line_locate_point(PG_FUNCTION_ARGS)
 	}
 	if ( gserialized_get_type(geom2) != POINTTYPE )
 	{
-		elog(ERROR,"line_locate_point: 2st arg isn't a point");
+		elog(ERROR,"line_locate_point: 2nd arg isn't a point");
 		PG_RETURN_NULL();
 	}
 
-	error_if_srid_mismatch(gserialized_get_srid(geom1), gserialized_get_srid(geom2));
+	gserialized_error_if_srid_mismatch(geom1, geom2, __func__);
 
 	lwline = lwgeom_as_lwline(lwgeom_from_gserialized(geom1));
 	lwpoint = lwgeom_as_lwpoint(lwgeom_from_gserialized(geom2));
