@@ -132,6 +132,8 @@ Datum LWGEOM_in(PG_FUNCTION_ARGS)
 		unsigned char *wkb = bytes_from_hexbytes(str, hexsize);
 		/* TODO: 20101206: No parser checks! This is inline with current 1.5 behavior, but needs discussion */
 		lwgeom = lwgeom_from_wkb(wkb, hexsize/2, LW_PARSER_CHECK_NONE);
+		/* Parser should throw error, but if not, catch here. */
+		if ( !lwgeom ) PG_RETURN_NULL();
 		/* If we picked up an SRID at the head of the WKB set it manually */
 		if ( srid ) lwgeom_set_srid(lwgeom, srid);
 		/* Add a bbox if necessary */
@@ -222,6 +224,11 @@ Datum LWGEOM_to_latlon(PG_FUNCTION_ARGS)
     PG_RETURN_NULL();
   }
 
+	if (!lwgeom_isfinite(lwgeom)) {
+    lwpgerror("ST_AsLatLonText: invalid coordinate");
+    PG_RETURN_NULL();
+  }
+
 	format_str = text_to_cstring(format_text);
   assert(format_str != NULL);
 
@@ -760,6 +767,11 @@ Datum LWGEOM_recv(PG_FUNCTION_ARGS)
 	}
 
 	lwgeom = lwgeom_from_wkb((uint8_t*)buf->data, buf->len, LW_PARSER_CHECK_ALL);
+	if ( !lwgeom )
+	{
+		ereport(ERROR,(errmsg("recv error - invalid geometry")));
+		PG_RETURN_NULL();
+	}
 
 	if ( lwgeom_needs_bbox(lwgeom) )
 		lwgeom_add_bbox(lwgeom);
