@@ -39,9 +39,9 @@
 
 #include "../../postgis_config.h"
 
-#if POSTGIS_PGSQL_VERSION > 92
+
 #include "access/htup_details.h" /* for heap_form_tuple() */
-#endif
+
 
 #include "rtpostgis.h"
 
@@ -64,6 +64,8 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS);
 Datum RASTER_valueCount(PG_FUNCTION_ARGS);
 Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS);
 
+#define VALUES_LENGTH 6
+
 /**
  * Get summary stats of a band
  */
@@ -80,9 +82,8 @@ Datum RASTER_summaryStats(PG_FUNCTION_ARGS)
 	rt_bandstats stats = NULL;
 
 	TupleDesc tupdesc;
-	int values_length = 6;
-	Datum values[values_length];
-	bool nulls[values_length];
+	Datum values[VALUES_LENGTH];
+	bool nulls[VALUES_LENGTH];
 	HeapTuple tuple;
 	Datum result;
 
@@ -160,7 +161,7 @@ Datum RASTER_summaryStats(PG_FUNCTION_ARGS)
 
 	BlessTupleDesc(tupdesc);
 
-	memset(nulls, FALSE, sizeof(bool) * values_length);
+	memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 	values[0] = Int64GetDatum(stats->count);
 	if (stats->count > 0) {
@@ -224,9 +225,8 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 	rt_bandstats stats = NULL;
 	rt_bandstats rtn = NULL;
 
-	int values_length = 6;
-	Datum values[values_length];
-	bool nulls[values_length];
+	Datum values[VALUES_LENGTH];
+	bool nulls[VALUES_LENGTH];
 	Datum result;
 
 	/* tablename is null, return null */
@@ -309,13 +309,11 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 	SPI_cursor_fetch(portal, TRUE, 1);
 	while (SPI_processed == 1 && SPI_tuptable != NULL) {
 		tupdesc = SPI_tuptable->tupdesc;
-		tuptable = SPI_tuptable;
-		tuple = tuptable->vals[0];
+		tuple = SPI_tuptable->vals[0];
 
 		datum = SPI_getbinval(tuple, tupdesc, 1, &isNull);
 		if (SPI_result == SPI_ERROR_NOATTRIBUTE) {
-
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			SPI_freetuptable(SPI_tuptable);
 			SPI_cursor_close(portal);
 			SPI_finish();
 
@@ -332,8 +330,7 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 
 		raster = rt_raster_deserialize(pgraster, FALSE);
 		if (!raster) {
-
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			SPI_freetuptable(SPI_tuptable);
 			SPI_cursor_close(portal);
 			SPI_finish();
 
@@ -349,7 +346,7 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 
 			rt_raster_destroy(raster);
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			SPI_freetuptable(SPI_tuptable);
 			SPI_cursor_close(portal);
 			SPI_finish();
 
@@ -364,7 +361,7 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 
 			rt_raster_destroy(raster);
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			SPI_freetuptable(SPI_tuptable);
 			SPI_cursor_close(portal);
 			SPI_finish();
 
@@ -381,7 +378,7 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 		if (NULL == stats) {
 			elog(NOTICE, "Cannot compute summary statistics for band at index %d. Returning NULL", bandindex);
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			SPI_freetuptable(SPI_tuptable);
 			SPI_cursor_close(portal);
 			SPI_finish();
 
@@ -394,8 +391,7 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 			if (NULL == rtn) {
 				rtn = (rt_bandstats) SPI_palloc(sizeof(struct rt_bandstats_t));
 				if (NULL == rtn) {
-
-					if (SPI_tuptable) SPI_freetuptable(tuptable);
+					SPI_freetuptable(SPI_tuptable);
 					SPI_cursor_close(portal);
 					SPI_finish();
 
@@ -431,7 +427,7 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 		SPI_cursor_fetch(portal, TRUE, 1);
 	}
 
-	if (SPI_tuptable) SPI_freetuptable(tuptable);
+	if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 	SPI_cursor_close(portal);
 	SPI_finish();
 
@@ -462,7 +458,7 @@ Datum RASTER_summaryStatsCoverage(PG_FUNCTION_ARGS)
 
 	BlessTupleDesc(tupdesc);
 
-	memset(nulls, FALSE, sizeof(bool) * values_length);
+	memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 	values[0] = Int64GetDatum(rtn->count);
 	if (rtn->count > 0) {
@@ -833,9 +829,8 @@ Datum RASTER_summaryStats_finalfn(PG_FUNCTION_ARGS)
 
 	TupleDesc tupdesc;
 	HeapTuple tuple;
-	int values_length = 6;
-	Datum values[values_length];
-	bool nulls[values_length];
+	Datum values[VALUES_LENGTH];
+	bool nulls[VALUES_LENGTH];
 	Datum result;
 
 	POSTGIS_RT_DEBUG(3, "Starting...");
@@ -882,7 +877,7 @@ Datum RASTER_summaryStats_finalfn(PG_FUNCTION_ARGS)
 
 	BlessTupleDesc(tupdesc);
 
-	memset(nulls, FALSE, sizeof(bool) * values_length);
+	memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 	values[0] = Int64GetDatum(state->stats->count);
 	if (state->stats->count > 0) {
@@ -907,11 +902,18 @@ Datum RASTER_summaryStats_finalfn(PG_FUNCTION_ARGS)
 	result = HeapTupleGetDatum(tuple);
 
 	/* clean up */
-	rtpg_summarystats_arg_destroy(state);
+	/* For Windowing functions, it is important to leave */
+	/* the state intact, knowing that the aggcontext will be */
+	/* freed by PgSQL when the statement is complete. */
+	/* https://trac.osgeo.org/postgis/ticket/4770 */
+	// rtpg_summarystats_arg_destroy(state);
 
 	PG_RETURN_DATUM(result);
 }
 
+#undef VALUES_LENGTH
+#define VALUES_LENGTH 4
+
 /**
  * Returns histogram for a band
  */
@@ -1113,7 +1115,7 @@ Datum RASTER_histogram(PG_FUNCTION_ARGS)
 		}
 
 		/* get histogram */
-		hist = rt_band_get_histogram(stats, bin_count, bin_width, bin_width_count, right, min, max, &count);
+		hist = rt_band_get_histogram(stats, (uint32_t)bin_count, bin_width, bin_width_count, right, min, max, &count);
 		if (bin_width_count) pfree(bin_width);
 		pfree(stats);
 		if (NULL == hist || !count) {
@@ -1157,15 +1159,14 @@ Datum RASTER_histogram(PG_FUNCTION_ARGS)
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 4;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 		HeapTuple tuple;
 		Datum result;
 
 		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = Float8GetDatum(hist2[call_cntr].min);
 		values[1] = Float8GetDatum(hist2[call_cntr].max);
@@ -1196,7 +1197,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 	FuncCallContext *funcctx;
 	TupleDesc tupdesc;
 
-	int i;
+	uint32_t i;
 	rt_histogram covhist = NULL;
 	rt_histogram covhist2;
 	int call_cntr;
@@ -1229,7 +1230,6 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 		double max = 0;
 		int spi_result;
 		Portal portal;
-		SPITupleTable *tuptable = NULL;
 		HeapTuple tuple;
 		Datum datum;
 		bool isNull = FALSE;
@@ -1340,7 +1340,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 				&nulls, &n);
 
 			bin_width = palloc(sizeof(double) * n);
-			for (i = 0, j = 0; i < n; i++) {
+			for (i = 0, j = 0; i < (uint32_t) n; i++) {
 				if (nulls[i]) continue;
 
 				switch (etype) {
@@ -1391,7 +1391,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 		sql = (char *) palloc(len);
 		if (NULL == sql) {
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 			SPI_finish();
 
 			if (bin_width_count) pfree(bin_width);
@@ -1408,7 +1408,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 		pfree(sql);
 		if (spi_result != SPI_OK_SELECT || SPI_tuptable == NULL || SPI_processed != 1) {
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 			SPI_finish();
 
 			if (bin_width_count) pfree(bin_width);
@@ -1419,13 +1419,12 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 		}
 
 		tupdesc = SPI_tuptable->tupdesc;
-		tuptable = SPI_tuptable;
-		tuple = tuptable->vals[0];
+		tuple = SPI_tuptable->vals[0];
 
 		tmp = SPI_getvalue(tuple, tupdesc, 1);
 		if (NULL == tmp || !strlen(tmp)) {
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			SPI_freetuptable(SPI_tuptable);
 			SPI_finish();
 
 			if (bin_width_count) pfree(bin_width);
@@ -1441,7 +1440,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 		tmp = SPI_getvalue(tuple, tupdesc, 2);
 		if (NULL == tmp || !strlen(tmp)) {
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 			SPI_finish();
 
 			if (bin_width_count) pfree(bin_width);
@@ -1460,7 +1459,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 		sql = (char *) palloc(len);
 		if (NULL == sql) {
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 			SPI_finish();
 
 			if (bin_width_count) pfree(bin_width);
@@ -1486,13 +1485,11 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 		SPI_cursor_fetch(portal, TRUE, 1);
 		while (SPI_processed == 1 && SPI_tuptable != NULL) {
 			tupdesc = SPI_tuptable->tupdesc;
-			tuptable = SPI_tuptable;
-			tuple = tuptable->vals[0];
+			tuple = SPI_tuptable->vals[0];
 
 			datum = SPI_getbinval(tuple, tupdesc, 1, &isNull);
 			if (SPI_result == SPI_ERROR_NOATTRIBUTE) {
-
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -1513,7 +1510,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 			raster = rt_raster_deserialize(pgraster, FALSE);
 			if (!raster) {
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -1532,7 +1529,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 
 				rt_raster_destroy(raster);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -1550,7 +1547,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 
 				rt_raster_destroy(raster);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -1570,7 +1567,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 			if (NULL == stats) {
 				elog(NOTICE, "Cannot compute summary statistics for band at index %d. Returning NULL", bandindex);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -1588,7 +1585,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 				if (NULL == hist || !count) {
 					elog(NOTICE, "Cannot compute histogram for band at index %d", bandindex);
 
-					if (SPI_tuptable) SPI_freetuptable(tuptable);
+					if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 					SPI_cursor_close(portal);
 					SPI_finish();
 
@@ -1607,7 +1604,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 					if (NULL == covhist) {
 
 						pfree(hist);
-						if (SPI_tuptable) SPI_freetuptable(tuptable);
+						if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 						SPI_cursor_close(portal);
 						SPI_finish();
 
@@ -1643,7 +1640,7 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 			SPI_cursor_fetch(portal, TRUE, 1);
 		}
 
-		if (SPI_tuptable) SPI_freetuptable(tuptable);
+		if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 		SPI_cursor_close(portal);
 		SPI_finish();
 
@@ -1688,15 +1685,14 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 4;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 		HeapTuple tuple;
 		Datum result;
 
 		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = Float8GetDatum(covhist2[call_cntr].min);
 		values[1] = Float8GetDatum(covhist2[call_cntr].max);
@@ -1718,6 +1714,9 @@ Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
 	}
 }
 
+#undef VALUES_LENGTH
+#define VALUES_LENGTH 2
+
 /**
  * Returns quantiles for a band
  */
@@ -1940,15 +1939,14 @@ Datum RASTER_quantile(PG_FUNCTION_ARGS)
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 2;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 		HeapTuple tuple;
 		Datum result;
 
 		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = Float8GetDatum(quant2[call_cntr].quantile);
 		values[1] = Float8GetDatum(quant2[call_cntr].value);
@@ -1977,7 +1975,7 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 	FuncCallContext *funcctx;
 	TupleDesc tupdesc;
 
-	int i;
+	uint32_t i;
 	rt_quantile covquant = NULL;
 	rt_quantile covquant2;
 	int call_cntr;
@@ -2111,7 +2109,7 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 				&nulls, &n);
 
 			quantiles = palloc(sizeof(double) * n);
-			for (i = 0, j = 0; i < n; i++) {
+			for (i = 0, j = 0; i < (uint32_t) n; i++) {
 				if (nulls[i]) continue;
 
 				switch (etype) {
@@ -2228,13 +2226,11 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 			if (NULL != covquant) pfree(covquant);
 
 			tupdesc = SPI_tuptable->tupdesc;
-			tuptable = SPI_tuptable;
-			tuple = tuptable->vals[0];
+			tuple = SPI_tuptable->vals[0];
 
 			datum = SPI_getbinval(tuple, tupdesc, 1, &isNull);
 			if (SPI_result == SPI_ERROR_NOATTRIBUTE) {
-
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2252,7 +2248,7 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 			raster = rt_raster_deserialize(pgraster, FALSE);
 			if (!raster) {
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2268,7 +2264,7 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 
 				rt_raster_destroy(raster);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2283,7 +2279,7 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 
 				rt_raster_destroy(raster);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2302,10 +2298,10 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 			rt_band_destroy(band);
 			rt_raster_destroy(raster);
 
-			if (NULL == covquant || !count) {
+			if (!covquant || !count) {
 				elog(NOTICE, "Cannot compute quantiles for band at index %d", bandindex);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2325,10 +2321,10 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 				covquant2[i].value = covquant[i].value;
 		}
 
-		if (NULL != covquant) pfree(covquant);
+		pfree(covquant);
 		quantile_llist_destroy(&qlls, qlls_count);
 
-		if (SPI_tuptable) SPI_freetuptable(tuptable);
+		if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 		SPI_cursor_close(portal);
 		SPI_finish();
 
@@ -2369,15 +2365,14 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 2;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 		HeapTuple tuple;
 		Datum result;
 
 		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = Float8GetDatum(covquant2[call_cntr].quantile);
 		if (covquant2[call_cntr].has_value)
@@ -2401,6 +2396,9 @@ Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
 	}
 }
 
+#undef VALUES_LENGTH
+#define VALUES_LENGTH 3
+
 /* get counts of values */
 PG_FUNCTION_INFO_V1(RASTER_valueCount);
 Datum RASTER_valueCount(PG_FUNCTION_ARGS) {
@@ -2583,15 +2581,14 @@ Datum RASTER_valueCount(PG_FUNCTION_ARGS) {
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 3;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 		HeapTuple tuple;
 		Datum result;
 
 		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = Float8GetDatum(vcnts2[call_cntr].value);
 		values[1] = UInt32GetDatum(vcnts2[call_cntr].count);
@@ -2618,7 +2615,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 	FuncCallContext *funcctx;
 	TupleDesc tupdesc;
 
-	int i;
+	uint32_t i;
 	uint64_t covcount = 0;
 	uint64_t covtotal = 0;
 	rt_valuecount covvcnts = NULL;
@@ -2646,7 +2643,6 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 		char *sql = NULL;
 		int spi_result;
 		Portal portal;
-		SPITupleTable *tuptable = NULL;
 		HeapTuple tuple;
 		Datum datum;
 		bool isNull = FALSE;
@@ -2659,7 +2655,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 		rt_valuecount vcnts = NULL;
 		int exists = 0;
 
-		int j;
+		uint32_t j;
 		int n;
 
 		ArrayType *array;
@@ -2735,7 +2731,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 				&nulls, &n);
 
 			search_values = palloc(sizeof(double) * n);
-			for (i = 0, j = 0; i < n; i++) {
+			for (i = 0, j = 0; i < (uint32_t) n; i++) {
 				if (nulls[i]) continue;
 
 				switch (etype) {
@@ -2781,7 +2777,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 		sql = (char *) palloc(len);
 		if (NULL == sql) {
 
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
+			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 			SPI_finish();
 
 			if (search_values_count) pfree(search_values);
@@ -2807,13 +2803,12 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 		SPI_cursor_fetch(portal, TRUE, 1);
 		while (SPI_processed == 1 && SPI_tuptable != NULL) {
 			tupdesc = SPI_tuptable->tupdesc;
-			tuptable = SPI_tuptable;
-			tuple = tuptable->vals[0];
+			tuple = SPI_tuptable->vals[0];
 
 			datum = SPI_getbinval(tuple, tupdesc, 1, &isNull);
 			if (SPI_result == SPI_ERROR_NOATTRIBUTE) {
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2834,7 +2829,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 			raster = rt_raster_deserialize(pgraster, FALSE);
 			if (!raster) {
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2853,7 +2848,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 
 				rt_raster_destroy(raster);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2871,7 +2866,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 
 				rt_raster_destroy(raster);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2889,7 +2884,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 			if (NULL == vcnts || !count) {
 				elog(NOTICE, "Cannot count the values for band at index %d", bandindex);
 
-				if (SPI_tuptable) SPI_freetuptable(tuptable);
+				SPI_freetuptable(SPI_tuptable);
 				SPI_cursor_close(portal);
 				SPI_finish();
 
@@ -2906,7 +2901,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 				covvcnts = (rt_valuecount) SPI_palloc(sizeof(struct rt_valuecount_t) * count);
 				if (NULL == covvcnts) {
 
-					if (SPI_tuptable) SPI_freetuptable(tuptable);
+					SPI_freetuptable(SPI_tuptable);
 					SPI_cursor_close(portal);
 					SPI_finish();
 
@@ -2942,14 +2937,12 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 					else {
 						covcount++;
 						covvcnts = SPI_repalloc(covvcnts, sizeof(struct rt_valuecount_t) * covcount);
-						if (NULL == covvcnts) {
-
-							if (SPI_tuptable) SPI_freetuptable(tuptable);
+						if (!covvcnts) {
+							SPI_freetuptable(SPI_tuptable);
 							SPI_cursor_close(portal);
 							SPI_finish();
 
 							if (search_values_count) pfree(search_values);
-							if (NULL != covvcnts) free(covvcnts);
 
 							MemoryContextSwitchTo(oldcontext);
 							elog(ERROR, "RASTER_valueCountCoverage: Cannot change allocated memory for value counts of coverage");
@@ -2971,7 +2964,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 			SPI_cursor_fetch(portal, TRUE, 1);
 		}
 
-		if (SPI_tuptable) SPI_freetuptable(tuptable);
+		if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
 		SPI_cursor_close(portal);
 		SPI_finish();
 
@@ -3015,15 +3008,14 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 3;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 		HeapTuple tuple;
 		Datum result;
 
 		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = Float8GetDatum(covvcnts2[call_cntr].value);
 		values[1] = UInt32GetDatum(covvcnts2[call_cntr].count);
